@@ -15,23 +15,16 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import asyncio
-import pickle
 import re
-import unicodedata
 from datetime import datetime, time, timedelta
 from hashlib import md5, sha256
-from pathlib import Path
 from random import randint
-from typing import Any, Callable, ClassVar, List, MutableMapping, Optional
+from typing import Any, Callable, ClassVar, List, MutableMapping, Optional, Tuple
 
-from aiopath import AsyncPath
-from pymongo.errors import DuplicateKeyError
 from pyrogram.errors import (
     ChatAdminRequired,
     FloodWait,
     MessageDeleteForbidden,
-    MessageIdInvalid,
-    MessageNotModified,
     PeerIdInvalid,
     QueryIdInvalid,
     UserAdminInvalid,
@@ -45,56 +38,33 @@ from pyrogram.types import (
 )
 
 try:
-    from scipy.stats import ttest_1samp
-    from sklearn.pipeline import Pipeline
+    from userbotindo import Classifier
 
     _run_predict = True
 except ImportError:
-    from anjani.util.types import Pipeline
+    from anjani.util.types import Classifier
 
     _run_predict = False
 
 from anjani import command, filters, listener, plugin, util
 from anjani.util.misc import StopPropagation
 
-env = Path("config.env")
-try:
-    token = re.search(r'^(?!#)\s+?SP_TOKEN="(\w+)"', env.read_text().strip(), re.MULTILINE).group(  # type: ignore
-        1
-    )
-except (AttributeError, FileNotFoundError):
-    token = ""
-
-del env
-
-
-def get_trust(sample: List[float]) -> Optional[float]:
-    """Compute the trust score of a user
-    Args:
-        sample (List[float]): A list of scores
-    Returns:
-        Optional[float]: The trust score of the user
-    """
-    if not _run_predict:
-        return None
-    if len(sample) < 3:
-        return None  # Not enough data
-    _, pred = ttest_1samp(sample, 0.5, alternative="greater")
-    return pred * 100
-
 
 class SpamPrediction(plugin.Plugin):
     name: ClassVar[str] = "SpamPredict"
     helpable: ClassVar[bool] = True
-    disabled: ClassVar[bool] = not _run_predict or not token
+    disabled: ClassVar[bool] = not _run_predict
 
     db: util.db.AsyncCollection
     user_db: util.db.AsyncCollection
     setting_db: util.db.AsyncCollection
-    model: Pipeline
-    __min_reputation: int = 300
+    model: Classifier
+
+    __predict_cost: int = 10
+    __log_channel: int = -1001314588569
 
     async def on_load(self) -> None:
+        self.model = Classifier()
         self.db = self.bot.db.get_collection("SPAM_DUMP")
         self.user_db = self.bot.db.get_collection("USERS")
         self.setting_db = self.bot.db.get_collection("SPAM_PREDICT_SETTING")
@@ -131,15 +101,11 @@ class SpamPrediction(plugin.Plugin):
 
     async def __load_model(self) -> None:
         self.log.info("Downloading spam prediction model!")
-        async with self.bot.http.post(
-            "https://spamdetect.userbotindo.com",
-            headers={"Authorization": f"Bearer {token}"},
-        ) as res:
-            if res.status == 200:
-                self.model = await util.run_sync(pickle.loads, await res.read())
-            else:
-                self.log.warning("Failed to download prediction model!")
-                self.bot.unload_plugin(self)
+        try:
+            await self.model.load_model(self.bot.http)
+        except RuntimeError:
+            self.log.warning("Failed to download prediction model!")
+            self.bot.unload_plugin(self)
 
     @staticmethod
     def _build_hash(content: str) -> str:
@@ -151,23 +117,6 @@ class SpamPrediction(plugin.Plugin):
         # skipcq: PTC-W1003
         return md5((str(id) + self.bot.user.username).encode()).hexdigest()  # skipcq: BAN-B324
 
-    @staticmethod
-    def _normalize_text(text: str) -> str:
-        """Normalize text to remove accents and other non-ASCII characters."""
-        return (
-            unicodedata.normalize("NFKD", text).encode("utf-8", "ignore").decode("utf-8", "ignore")
-        ).lower()
-
-    @staticmethod
-    def prob_to_string(value: float) -> str:
-        return str(value * 10**2)[0:7]
-
-    async def _predict(self, text: str) -> util.types.NDArray[float]:
-        return await util.run_sync(self.model.predict_proba, [text])
-
-    async def _is_spam(self, text: str) -> bool:
-        return (await util.run_sync(self.model.predict, [text]))[0] == "spam"
-
     async def _collect_random_sample(self, proba: float, uid: Optional[int]) -> None:
         if not uid or uid == self.bot.uid:
             return
@@ -184,33 +133,6 @@ class SpamPrediction(plugin.Plugin):
                 },
             )  # Do not upsert
 
-    async def run_ocr(self, message: Message) -> Optional[str]:
-        """Run tesseract"""
-        try:
-            image = AsyncPath(await message.download())
-        except Exception:  # skipcq: PYL-W0703
-            return self.log.warning(
-                "Failed to download image from MessageID %s in Chat %s",
-                message.id,
-                message.chat.id,
-            )
-
-        try:
-            stdout, _, exitCode = await util.system.run_command(
-                "tesseract", str(image), "stdout", "-l", "eng+ind", "--psm", "6"
-            )
-        except FileNotFoundError:
-            return
-        except Exception as e:  # skipcq: PYL-W0703
-            return self.log.error("Unexpected error occured when running OCR", exc_info=e)
-        finally:
-            await image.unlink()
-
-        if exitCode != 0:
-            return self.log.warning("tesseract returned code '%s', %s", exitCode, stdout)
-
-        return stdout
-
     @listener.filters(
         filters.regex(r"spam_check_(?P<value>t|f)") | filters.regex(r"spam_ban_(?P<user>.*)")
     )
@@ -276,7 +198,9 @@ class SpamPrediction(plugin.Plugin):
 
         data = await self.db.find_one({"_id": content_hash})
         if not data:
-            return await query.answer("The voting poll for this message has ended!")
+            return await query.answer(
+                "The voting poll for this message has ended!", show_alert=True
+            )
 
         users_on_correct = data["spam"]
         users_on_incorrect = data["ham"]
@@ -336,30 +260,9 @@ class SpamPrediction(plugin.Plugin):
             if len(old_btn) > 1:
                 button.append(old_btn[1])
 
-        for i in data["msg_id"]:
-            while True:
-                try:
-                    await self.bot.client.edit_message_reply_markup(
-                        -1001314588569, i, InlineKeyboardMarkup(button)
-                    )
-                except MessageIdInvalid:
-                    pass
-                except MessageNotModified:
-                    await query.answer(
-                        "You already voted this content, "
-                        "this happened because there are multiple same of contents exists.",
-                        show_alert=True,
-                    )
-                except FloodWait as flood:
-                    await query.answer(
-                        "Please wait i'm updating the content for you.",
-                        show_alert=True,
-                    )
-                    await asyncio.sleep(flood.value)  # type: ignore
-                    continue
-
-                await asyncio.sleep(0.1)
-                break
+        await self.bot.client.edit_message_reply_markup(
+            self.__log_channel, data["msg_id"], InlineKeyboardMarkup(button)
+        )
 
         await query.answer()
 
@@ -376,42 +279,66 @@ class SpamPrediction(plugin.Plugin):
             if message.text
             else (message.caption if message.media and message.caption else None)
         )
-        task = None
-        if message.photo:
-            ocr_result = await self.run_ocr(message)
-            if ocr_result:
-                task = self.bot.loop.create_task(
-                    self.spam_check(message, ocr_result, from_ocr=True)
-                )
-
         if not chat or message.left_chat_member:
             return
 
         if not text:
-            if task and not task.done():
-                await task
-
             return
 
         # Always check the spam probability
-        try:
-            await self.spam_check(message, text)
-        finally:
-            if task and not task.done():
-                await task
+        await self.spam_check(message, text)
 
-    async def spam_check(self, message: Message, text: str, *, from_ocr: bool = False) -> None:
+    async def _build_notice(
+        self, message: Message, text: str, proba_str: str, identifier: str, content_hash: str
+    ) -> Tuple[str, List[List[InlineKeyboardButton]]]:
+        notice = (
+            "#SPAM_PREDICTION\n\n"
+            f"**Prediction Result**: {proba_str}\n"
+            f"**Identifier**: `{identifier}`\n"
+        )
+        if ch := message.forward_from_chat:
+            notice += f"**Channel ID**: `{self._build_hex(ch.id)}`\n"
+
+        notice += f"**Message Text Hash**: `{content_hash}`\n\n**====== CONTENT =======**\n\n{text}"
+        l_spam, l_ham = 0, 0
+        _, data = await asyncio.gather(
+            self.bot.log_stat("predicted"), self.db.find_one({"_id": content_hash})
+        )
+        if data:
+            l_spam = len(data["spam"])
+            l_ham = len(data["ham"])
+
+        keyb = [
+            [
+                InlineKeyboardButton(text=f"✅ Correct ({l_spam})", callback_data="spam_check_t"),
+                InlineKeyboardButton(text=f"❌ Incorrect ({l_ham})", callback_data="spam_check_f"),
+            ],
+            [InlineKeyboardButton(text="Chat", url=f"https://t.me/{message.chat.username}")],
+        ]
+
+        if message.forward_from_chat and message.forward_from_chat.username:
+            raw_btn = InlineKeyboardButton(
+                text="Channel", url=f"https://t.me/{message.forward_from_chat.username}"
+            )
+            if message.chat.username:
+                keyb[1].append(raw_btn)
+            else:
+                keyb.append([raw_btn])
+
+        return notice, keyb
+
+    async def spam_check(self, message: Message, text: str) -> None:
         text = text.strip()
         try:
             user = message.from_user.id
         except AttributeError:
             user = None
 
-        text_norm = self._normalize_text(text)
-        if len(text_norm.split()) < 5:  # Skip short messages
+        text_norm = self.model.normalize(text)
+        if len(text_norm.split()) < 4:  # Skip short messages
             return
 
-        response = await self._predict(text_norm)
+        response = await self.model.predict(text_norm)
         if response.size == 0:
             return
 
@@ -424,86 +351,48 @@ class SpamPrediction(plugin.Plugin):
 
         content_hash = self._build_hash(text)
         identifier = self._build_hex(user)
-        proba_str = self.prob_to_string(probability)
-        msg = None
+        proba_str = self.model.prob_to_string(probability)
+        msg_id = None
 
+        # only log public chat
         if message.chat.username:
-
-            notice = (
-                "#SPAM_PREDICTION\n\n"
-                f"**Prediction Result**: {proba_str}\n"
-                f"**Identifier**: `{identifier}`\n"
+            notice, keyb = await self._build_notice(
+                message, text, proba_str, identifier, content_hash
             )
-            if ch := message.forward_from_chat:
-                notice += f"**Channel ID**: `{self._build_hex(ch.id)}`\n"
 
-            if from_ocr:
-                notice += (
-                    f"**Photo Text Hash**: `{content_hash}`\n\n**====== CONTENT =======**\n\n{text}"
-                )
-            else:
-                notice += f"**Message Text Hash**: `{content_hash}`\n\n**====== CONTENT =======**\n\n{text}"
-
-            l_spam, l_ham = 0, 0
-            _, data = await asyncio.gather(
-                self.bot.log_stat("predicted"), self.db.find_one({"_id": content_hash})
-            )
-            if data:
-                l_spam = len(data["spam"])
-                l_ham = len(data["ham"])
-
-            keyb = [
-                [
-                    InlineKeyboardButton(
-                        text=f"✅ Correct ({l_spam})", callback_data="spam_check_t"
-                    ),
-                    InlineKeyboardButton(
-                        text=f"❌ Incorrect ({l_ham})", callback_data="spam_check_f"
-                    ),
-                ],
-                [InlineKeyboardButton(text="Chat", url=f"https://t.me/{message.chat.username}")],
-            ]
-
-            if message.forward_from_chat and message.forward_from_chat.username:
-                raw_btn = InlineKeyboardButton(
-                    text="Channel", url=f"https://t.me/{message.forward_from_chat.username}"
-                )
-                if message.chat.username:
-                    keyb[1].append(raw_btn)
+            async with asyncio.Lock():
+                data = await self.db.find_one({"_id": content_hash})
+                if data:
+                    msg_id = data["msg_id"]
                 else:
-                    keyb.append([raw_btn])
+                    while True:
+                        try:
+                            msg = await self.bot.client.send_message(
+                                chat_id=self.__log_channel,
+                                text=notice,
+                                disable_web_page_preview=True,
+                                reply_markup=InlineKeyboardMarkup(keyb),
+                            )
+                            msg_id = msg.id
+                        except FloodWait as flood:
+                            await asyncio.sleep(flood.value)  # type: ignore
+                            continue
 
-            while True:
-                try:
-                    msg = await self.bot.client.send_message(
-                        chat_id=-1001314588569,
-                        text=notice,
-                        disable_web_page_preview=True,
-                        reply_markup=InlineKeyboardMarkup(keyb),
-                    )
-                except FloodWait as flood:
-                    await asyncio.sleep(flood.value)  # type: ignore
-                    continue
+                        await asyncio.sleep(0.1)
+                        break
 
-                await asyncio.sleep(0.1)
-                break
-
-            try:
-                async with asyncio.Lock():
                     await self.db.insert_one(
                         {
                             "_id": content_hash,
                             "user": identifier,
-                            "text": text_norm,
                             "spam": [],
                             "ham": [],
                             "proba": probability,
-                            "msg_id": [msg.id],
+                            "msg_id": msg.id,
                             "date": util.time.sec(),
-                        }
+                            "text": text_norm,
+                        },
                     )
-            except DuplicateKeyError:
-                await self.db.update_one({"_id": content_hash}, {"$push": {"msg_id": msg.id}})
 
         if probability >= 0.8:
             chat = message.chat
@@ -511,7 +400,7 @@ class SpamPrediction(plugin.Plugin):
                 if message.sender_chat.id == chat.id:  # anon admin
                     return
 
-                current_chat: Any = await self.bot.client.get_chat(chat.id)
+                current_chat: Any = await self.bot.get_chat(chat.id)
                 if (
                     current_chat.linked_chat
                     and message.sender_chat.id == current_chat.linked_chat.id
@@ -529,20 +418,12 @@ class SpamPrediction(plugin.Plugin):
                     if util.tg.is_staff_or_admin(target):
                         return
 
-            if from_ocr:
-                alert = (
-                    f"❗️**PHOTO SPAM ALERT**❗️\n\n"
-                    f"**User**: `{identifier}`\n"
-                    f"**Photo Text Hash**: `{content_hash}`\n"
-                    f"**Spam Probability**: `{proba_str}%`"
-                )
-            else:
-                alert = (
-                    f"❗️**MESSAGE SPAM ALERT**❗️\n\n"
-                    f"**User**: `{identifier}`\n"
-                    f"**Message Text Hash**: `{content_hash}`\n"
-                    f"**Spam Probability**: `{proba_str}%`"
-                )
+            alert = (
+                f"❗️**MESSAGE SPAM ALERT**❗️\n\n"
+                f"**User**: `{identifier}`\n"
+                f"**Message Text Hash**: `{content_hash}`\n"
+                f"**Spam Probability**: `{proba_str}%`"
+            )
 
             await self.bot.log_stat("spam_detected")
             try:
@@ -558,8 +439,14 @@ class SpamPrediction(plugin.Plugin):
             chat = message.chat
             button = []
             me = await chat.get_member(self.bot.uid)
-            if message.chat.username and msg:
-                button.append([InlineKeyboardButton("View Message", url=msg.link)])
+            if message.chat.username and msg_id:
+                button.append(
+                    [
+                        InlineKeyboardButton(
+                            "View Message", url=f"https://t.me/SpamPredictionLog/{msg_id}"
+                        )
+                    ]
+                )
 
             if me.privileges and me.privileges.can_restrict_members and target is not None:
                 button.append(
@@ -578,51 +465,6 @@ class SpamPrediction(plugin.Plugin):
             )
             raise StopPropagation
 
-    async def mark_spam_ocr(
-        self, content: str, user_id: Optional[int], chat_id: int, message_id: int
-    ) -> bool:
-        identifier = self._build_hex(user_id)
-        content_hash = self._build_hash(content)
-        pred = await self._predict(content)
-        if pred.size == 0:
-            return False
-
-        proba = pred[0][1]
-        text = f"#SPAM\n\n**CPU Prediction**: `{self.prob_to_string(proba)}`\n"
-        if identifier:
-            text += f"**Identifier**: `{identifier}`\n"
-
-        text += f"**Photo Text Hash**: `{content_hash}`\n\n**======= CONTENT =======**\n\n{content}"
-        res = await asyncio.gather(
-            self.bot.client.send_message(
-                chat_id=-1001314588569,
-                text=text,
-                disable_web_page_preview=True,
-            ),
-            self.db.update_one(
-                {"_id": content_hash},
-                {
-                    "$set": {
-                        "text": content,
-                        "spam": 1,
-                        "ham": 0,
-                    }
-                },
-                upsert=True,
-            ),
-            self.bot.log_stat("spam_detected"),
-            self.bot.log_stat("predicted"),
-        )
-        await self.bot.client.send_message(
-            chat_id,
-            "Message photo logged as spam!",
-            reply_markup=InlineKeyboardMarkup(
-                [[InlineKeyboardButton("View Message", url=res[0].link)]]
-            ),
-            reply_to_message_id=message_id,
-        )
-        return True
-
     @command.filters(filters.staff_only)
     async def cmd_update_model(self, ctx: command.Context) -> Optional[str]:
         await self.__load_model()
@@ -642,27 +484,18 @@ class SpamPrediction(plugin.Plugin):
         else:
             content = ctx.input
 
-        if reply_msg and reply_msg.photo:
-            ocr_result = await self.run_ocr(reply_msg)
-            if ocr_result:
-                await self.mark_spam_ocr(ocr_result, user_id, ctx.chat.id, reply_msg.id)
-
-                # Return early if content is empty, so error message not shown
-                if not content:
-                    return None
-
         if not content:
             return await ctx.get_text("spampredict-empty")
 
         identifier = self._build_hex(user_id)
         content_hash = self._build_hash(content)
-        content_normalized = self._normalize_text(content.strip())
-        pred = await self._predict(content_normalized)
+        content_normalized = self.model.normalize(content.strip())
+        pred = await self.model.predict(content_normalized)
         if pred.size == 0:
             return "Prediction failed"
 
         proba = pred[0][1]
-        text = f"#SPAM\n\n**CPU Prediction**: `{self.prob_to_string(proba)}`\n"
+        text = f"#SPAM\n\n**CPU Prediction**: `{self.model.prob_to_string(proba)}`\n"
         if identifier:
             text += f"**Identifier**: `{identifier}`\n"
 
@@ -680,7 +513,7 @@ class SpamPrediction(plugin.Plugin):
                 upsert=True,
             ),
             self.bot.client.send_message(
-                chat_id=-1001314588569,
+                chat_id=self.__log_channel,
                 text=text,
                 disable_web_page_preview=True,
             ),
@@ -703,12 +536,12 @@ class SpamPrediction(plugin.Plugin):
         if not user:
             return None
 
-        if user.get("reputation", 0) < self.__min_reputation:
+        if user.get("reputation", 0) < self.__predict_cost:
             return await self.text(
                 chat.id,
-                "spampredict-unauthorized",
+                "spampredict-insuficent",
+                self.__predict_cost,
                 user.get("reputation", 0),
-                self.__min_reputation,
             )
 
         replied = ctx.msg.reply_to_message
@@ -717,54 +550,26 @@ class SpamPrediction(plugin.Plugin):
             return None
 
         content = replied.text or replied.caption
-
-        photo_prediction = None
-        if replied.photo:
-            await ctx.respond(
-                await ctx.get_text("spampredict-photo"),
-                reply_to_message_id=replied.id,
-            )
-
-            ocr_result = await self.run_ocr(replied)
-            if ocr_result:
-                ocr_prediction = await self._predict(ocr_result)
-                if ocr_prediction.size != 0:
-                    photo_prediction = (
-                        "**Result Photo Text**\n\n"
-                        f"**Is Spam**: {await self._is_spam(ocr_result)}\n"
-                        f"**Spam Prediction**: `{self.prob_to_string(ocr_prediction[0][1])}`\n"
-                        f"**Ham Prediction**: `{self.prob_to_string(ocr_prediction[0][0])}`\n\n"
-                    )
-                    # Return early if content is empty, so error message not shown
-                    if not content:
-                        await asyncio.gather(
-                            self.bot.log_stat("predicted"),
-                            ctx.respond(photo_prediction),
-                        )
-                        return None
-            else:
-                photo_prediction = await ctx.get_text("spampredict-photo-failed")
-
         if not content:
             return await ctx.get_text("spampredict-empty")
-
-        content = self._normalize_text(content.strip())
-        pred = await self._predict(content)
+        content = self.model.normalize(content.strip())
+        pred = await self.model.predict(content)
         if pred.size == 0:
             return await ctx.get_text("spampredict-failed")
 
         textPrediction = (
-            f"**Is Spam**: {await self._is_spam(content)}\n"
-            f"**Spam Prediction**: `{self.prob_to_string(pred[0][1])}`\n"
-            f"**Ham Prediction**: `{self.prob_to_string(pred[0][0])}`"
+            f"**Is Spam**: {await self.model.is_spam(content)}\n"
+            f"**Spam Prediction**: `{self.model.prob_to_string(pred[0][1])}`\n"
+            f"**Ham Prediction**: `{self.model.prob_to_string(pred[0][0])}`"
         )
         await asyncio.gather(
             self.bot.log_stat("predicted"),
             ctx.respond(
-                photo_prediction + "**Result Caption Text**\n\n" + textPrediction
-                if photo_prediction
-                else "**Result**\n\n" + textPrediction,
-                reply_to_message_id=None if replied.photo else replied.id,
+                "**Result**\n\n" + textPrediction,
+                reply_to_message_id=replied.id,
+            ),
+            self.user_db.update_one(
+                {"_id": ctx.author.id}, {"$inc": {"reputation": -self.__predict_cost}}
             ),
         )
         return None
