@@ -29,7 +29,10 @@ from pyrogram.types import CallbackQuery, Chat, ChatPreview, Message, User
 
 from anjani import command, listener, plugin, util
 
-from .spam_prediction import get_trust
+try:
+    from userbotindo import get_trust
+except ImportError:
+    from anjani.util.misc import do_nothing as get_trust
 
 
 class Users(plugin.Plugin):
@@ -209,7 +212,7 @@ class Users(plugin.Plugin):
         user_db = await self.users_db.find_one({"_id": user.id})
         if user_db and self.predict_loaded:
             if user_db.get("spam", False):
-                text += "<b>\n⚠️Warning I flag this user as a scammer⚠️</b>\n"
+                text += "<b>\n⚠️Warning I flag this user as a spammer⚠️</b>\n"
 
             text += f"\n<b>Identifier:</b> <code>{user_db.get('hash', 'unknown')}</code>"
             text += f"\n<b>Reputation:</b> <code>{user_db.get('reputation', 0)}</code>"
