@@ -155,7 +155,11 @@ class Greeting(plugin.Plugin):
         first_name = user.first_name or ""  # Ensure first name is not None
         last_name = user.last_name
         full_name = first_name + last_name if last_name else first_name
-        count = await client.get_chat_members_count(chat.id) if client else "N/A"
+        try:
+            count = await client.get_chat_members_count(chat.id) if client else "N/A"
+        except ChannelPrivate:
+            count = "N/A"
+
         return text.format(
             first=escape(first_name),
             last=escape(last_name) if last_name else "",
@@ -261,7 +265,7 @@ class Greeting(plugin.Plugin):
 
         try:  # Try to build a text first to check message validity
             await self._build_text(welc_text or "", ctx.author, chat, self.bot.client)
-        except KeyError as e:
+        except (KeyError, ValueError) as e:
             return await self.text(chat.id, "err-msg-format-parsing", err=e)
 
         ret, _ = await asyncio.gather(
